package com.usky.fire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.fire.domain.MhBuild;
import com.usky.fire.domain.MhCompany;
import com.usky.fire.mapper.MhBuildMapper;
import com.usky.fire.mapper.MhCompanyMapper;
import com.usky.fire.service.MhCompanyService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.fire.service.vo.MhCompanyVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 闵行-单位信息表 服务实现类
 * </p>
 *
 * @author han
 * @since 2025-03-28
 */
@Service
public class MhCompanyServiceImpl extends AbstractCrudService<MhCompanyMapper, MhCompany> implements MhCompanyService {

    @Autowired
    private MhBuildMapper mhBuildMapper;

    @Override
    public CommonPage<MhCompanyVO> getWarehouse(Integer type, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        List<MhCompanyVO> returnList = new ArrayList<>();
        List<MhBuild> buildList = new ArrayList<>();
        List<MhCompany> companyList = new ArrayList<>();
        IPage<MhCompany> page = new Page<>(pageNum, pageSize);

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime start = null;
        try {
            start = LocalDateTime.parse(startTime, formatter);
        } catch (Exception e) {
            log.error("开始时间传参格式不正确，请重试！");
        }
        LocalDateTime end = null;
        try {
            end = LocalDateTime.parse(endTime, formatter);
        } catch (Exception e) {
            log.error("结束时间传参格式不正确，请重试！");
        }

        LambdaQueryWrapper<MhCompany> queryWrapper = Wrappers.lambdaQuery();

        switch (type) {
            case 1:
                queryWrapper
                        .and(wrapper -> wrapper
                                .like(MhCompany::getCompanyType1, "仓库")
                                .or()
                                .like(MhCompany::getCompanyType1, "厂房")
                                .or()
                                .like(MhCompany::getCompanyType3, "厂房")
                                .or()
                                .like(MhCompany::getCompanyType3, "仓库")
                        );
                break;
        }

        if (start != null && end != null) {
            queryWrapper.and((wrapper) -> {
                wrapper.ge(MhCompany::getCreateTime, startTime).le(MhCompany::getCreateTime, endTime);
            });
        } else if (start != null) {
            queryWrapper.and((wrapper) -> {
                wrapper.ge(MhCompany::getCreateTime, startTime);
            });
        } else if (end != null) {
            queryWrapper.and((wrapper) -> {
                wrapper.le(MhCompany::getCreateTime, endTime);
            });
        }

        queryWrapper.orderByDesc(MhCompany::getId);
        page = this.page(page, queryWrapper);

        companyList = page.getRecords();
        List<Long> buildIdList = page.getRecords().stream()
                .map(MhCompany::getBuildId).distinct().collect(Collectors.toList());

        if (!buildIdList.isEmpty()) {
            LambdaQueryWrapper<MhBuild> queryWrapper2 = Wrappers.lambdaQuery();
            queryWrapper2.in(MhBuild::getId, buildIdList);
            buildList = mhBuildMapper.selectList(queryWrapper2);
        }

        for (MhCompany company : companyList) {
            MhCompanyVO vo = new MhCompanyVO();
            vo.setMhCompany(company);
            for (MhBuild build : buildList) {
                if (build.getId().equals(company.getBuildId())) {
                    vo.setMhBuild(build);
                    break;
                }
            }
            returnList.add(vo);
        }

        return new CommonPage<>(returnList, page.getTotal(), pageSize, pageNum);
    }

}
